/*
 * Mon Jul 10 09:42:25 MET DST 1995
 *
 * CHORD2HTML is copyrighted 1995 by Pim Borst.
 * I allow the free redistribution of the original, unchanged file CRD2HTML.TGZ.
 * I do not take any responsibility for any damage caused by using or installing
 * this software.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <memory.h>

#define DIRSEP '/'
#define SPACEWIDTH 4
#define CHORDSPERLINE 8
#define MAXCHORDS 512
#define CHORDLEN 32

char used[MAXCHORDS][CHORDLEN];
int usedchords;
char defined[512][32];
int defchords;
int spacetab[256];
int spaces;
int song;
char chords[32][32];
char words[32][256];

char chorddir[256];
char spacedir[256];
char filename[256];

void print_line(char *s);
void enter_space(int l);
void make_spaces(void);
void enter_chord(char *s);
void convert_chord(char *s, char *d);
void print_chords(void);
void make_chord(char *s);
int find_chord(char *s);

FILE *in,*out;

void main(int argc,char *argv[])
{
  char s[256],*c,*e;
  int nochords,i,j;

  if (argc<1)
  {
 fprintf(stderr,"usage: chord2html [-s spacesdir] [-c chordsdir] chordfile\n");
    exit(-1);
  };

  strcpy(chorddir,"default.chords");
  strcpy(spacedir,".");

  i=1;
  while (i<argc && argv[i][0]=='-')
  {
    if (strcmp(argv[i],"-s")==0)
    {
      if (++i>=argc)
      {
        fprintf(stderr,"missing spaces directory\n");
        exit(-1);
      };
      strcpy(spacedir,argv[i++]);
    }
    else if (strcmp(argv[i],"-c")==0)
    {
      if (++i>=argc)
      {
        fprintf(stderr,"missing default chords directory\n");
        exit(-1);
      };
      strcpy(chorddir,argv[i++]);
    }
    else
    {
      fprintf(stderr,"unknown option %s\n",argv[i]);
      exit(-1);
    }
  };
  if (i>=argc)
  {
    fprintf(stderr,"missing chordfile\n");
    exit(-1);
  };
  if (i<argc-1)
  {
    fprintf(stderr,"too many arguments\n");
    exit(-1);
  };

  strcpy(filename,argv[i]);
  if ((in=fopen(filename,"r"))==NULL)
  {
    fprintf(stderr,"chord file %s inaccessible\n",filename);
    exit(-1);
  };

  i=strlen(filename);
  while (i>0 && filename[i]!='.' && filename[i]!=DIRSEP)
    i--;
  if (i>0 && filename[i]=='.')
    filename[i]='\0';

  sprintf(s,"%s.html",filename);
  if ((out=fopen(s,"w"))==NULL)
  {
    fprintf(stderr,"unable to create %s.html\n",filename);
    exit(-1);
  };

  usedchords=0;
  defchords=0;
  nochords=0;
  spaces=0;
  song=1;

  sprintf(s,"mkdir %s.chords%d",filename,song);
  system(s);

  while (fgets(s,255,in))
  {
    i=strlen(s)-1;
    if (i>=0 && s[i]=='\n')
      s[i]='\0';

    c=s;
    while (!nochords && (*c==' ' || *c=='\t'))
      c++;

    /* EMPTY LINE */
    if (!nochords && *c=='\0')
       fprintf(out,"<BR>\n");

    /* COMMENT LINE */
    else if (!nochords && *c=='#')
      /* nothing */ ;

    /* TITLE */
    else if ((c=strstr(s,"\{title:")) ||
        (c=strstr(s,"\{t:")))
    {
      c=strchr(c,':');
      c++;
      while (*c==' ')
        c++;
      if ((e=strchr(c,'}')))
        *e='\0';
      fprintf(out,"<H1 ALIGN=CENTER>%s</H1>\n",c);
    }

    /* SUBTITLE */
    else if ((c=strstr(s,"\{subtitle:")) ||
             (c=strstr(s,"\{st:")))
    {
      c=strchr(c,':');
      c++;
      while (*c==' ')
        c++;
      if ((e=strchr(c,'}')))
        *e='\0';
      fprintf(out,"<H2 ALIGN=CENTER>%s</H2>\n",c);
    }

    /* COMMENT */
    else if ((c=strstr(s,"\{comment:")) ||
             (c=strstr(s,"\{c:")))
    {
      c=strchr(c,':');
      c++;
      while (*c==' ')
        c++;
      if ((e=strchr(c,'}')))
        *e='\0';
      fprintf(out,"<P><B>%s</B>\n",c);
    }

    /* COMMENT ITALIC */
    else if ((c=strstr(s,"\{comment_italic:")) ||
             (c=strstr(s,"\{ci:")))
    {
      c=strchr(c,':');
      c++;
      while (*c==' ')
        c++;
      if ((e=strchr(c,'}')))
        *e='\0';
      fprintf(out,"<P><I>%s</I>\n",c);
    }

    /* COMMENT BOX */
    else if ((c=strstr(s,"\{comment_box:")) ||
             (c=strstr(s,"\{cb:")))
    {
      c=strchr(c,':');
      c++;
      while (*c==' ')
        c++;
      if ((e=strchr(c,'}')))
        *e='\0';
      fprintf(out,"<P><TABLE BORDER><TR><TD>%s</TR></TABLE>\n",c);
    }

    /* START OF CHORUS */
    else if ((c=strstr(s,"\{start_of_chorus")) ||
             (c=strstr(s,"\{soc")))
      fprintf(out,"<P><B>Chorus:</B><BLOCKQUOTE>\n");

    /* END OF CHORUS */
    else if ((c=strstr(s,"\{end_of_chorus")) ||
             (c=strstr(s,"\{eoc")))
      fprintf(out,"</BLOCKQUOTE>\n");

    /* START OF TAB */
    else if ((c=strstr(s,"\{start_of_tab")) ||
             (c=strstr(s,"\{sot")))
    {
      nochords=-1;
      fprintf(out,"<P><PRE>\n");
    }

    /* END OF TAB */
    else if ((c=strstr(s,"\{end_of_tab")) ||
             (c=strstr(s,"\{eot")))
    {
      nochords=0;
      fprintf(out,"</PRE>\n");
    }

    /* NEW SONG */
    else if ((c=strstr(s,"\{new_song")) ||
             (c=strstr(s,"\{ns")))
    {
      if (usedchords>0)
        print_chords();
      fprintf(out,"<HR>\n");
      song++;
      sprintf(s,"mkdir %s.chords%d",filename,song);
      system(s);
      defchords=0;
      usedchords=0;
    }

    /* NEW PAGE */
    else if ((c=strstr(s,"\{new_page")) ||
             (c=strstr(s,"\{new_physical_page")) ||
             (c=strstr(s,"\{np")) ||
             (c=strstr(s,"\{npp")))
      /* do nothing */;

    /* DEFINE */
    else if ((c=strstr(s,"\{define")))
      make_chord(c);

    /* TEXT LINE */
    else
    {
      if (nochords)
      /* TAB LINE */
        fprintf(out,"%s\n",s);
      else if (!strchr(s,'['))
      /* LINE WITHOUT CHORDS */
      {
        print_line(s);
        fprintf(out,"<BR>\n");
      }
      else
      /* LINE WITH CHORDS */
      {
        /* print chords */
        c=s;
        i=0;
        if ((e=strchr(c,'[')))
        {
          if (e!=c)
          {
            /* handle leading words */
            *e='\0';
            strcpy(chords[i],"\0");
            strcpy(words[i++],c);
            *e='[';
          };
          c=e;
        };
        while (*c)
        {
          if (*c=='[')
          {
            /* handle chord */
            if ((e=strchr(c,']')))
            {
              /* enter chord in line and dictionary */
              *e='\0';
              strcpy(chords[i],c+1);
              enter_chord(chords[i]);
              c=e+1;
            }
            else
            {
              /* enter chord in line and dictionary */
              strcpy(chords[i],c+1);
              enter_chord(chords[i]);
              c+=strlen(c);
            }
          };
          if ((e=strchr(c,'[')))
          {
            /* more chords to come */
            *e='\0';
            strcpy(words[i++],c);
            *e='[';
            c=e;
          }
          else
          {
            /* these are the last words */
            e=c+strlen(c);
            strcpy(words[i++],c);
            c=e;
          }
        };
        fprintf(out,"<P><TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0>\n<TR>");
        for (j=0; j<i; j++)
          fprintf(out,"<TD><I>%s</I>",chords[j]);
        fprintf(out,"</TR>\n<TR>");
        for (j=0; j<i; j++)
        {
          fprintf(out,"<TD>");
          print_line(words[j]);
        };
        fprintf(out,"</TR>\n</TABLE>\n");  
      }
    }
  };
  if (usedchords>0)
    print_chords();
  if (spaces>0)
    make_spaces();

  fclose(out);
  fclose(in);
}


void print_line(char *s)
{
  int l;

  if (s[0]==' ' && s[1]!=' ')
  {
     /* one leading space */
     fprintf(out,"<IMG SRC=\"%s%cspace1.gif\">",spacedir,DIRSEP);
     enter_space(1);
     s++;
  };
  l=0;
  while (*s)
  {
    if (*s==' ')
    {
      l++;
      s++;
    }
    else
    {
      if (l==1)
      {
        fprintf(out," ");
        l=0;
      }
      else if (l>1)
      {
        fprintf(out,"<IMG SRC=\"%s%cspace%d.gif\">",spacedir,DIRSEP,l);
        enter_space(l);
        l=0;
      };
      fprintf(out,"%c",*s++);
    }
  };
  if (l==1)
    fprintf(out," ");
  else if (l>1)
  {
    fprintf(out,"<IMG SRC=\"%s%cspace%d.gif\">",spacedir,DIRSEP,l);
    enter_space(l);
  }
}

void enter_space(int l)
{
  int i;

  i=0;
  while (i<spaces && spacetab[i]!=l)
    i++;
  if (i>=spaces || spacetab[i]!=l)
    spacetab[spaces++]=l;
}

void make_spaces(void)
{
  int i;
  char s[256];

  for (i=0; i<spaces; i++)
  {
    sprintf(s,"pbmmake -white %d 1 | ppmtogif -transparent rgb:ff/ff/ff > %s%cspace%d.gif",
              spacetab[i]*SPACEWIDTH,spacedir,DIRSEP,spacetab[i]);
    fprintf(stderr,"executing %s\n",s);
    system(s);
  }
}

void enter_chord(char *s)
{
  int i;

  i=0;
  while (i<usedchords && strcmp(s,used[i])!=0)
    i++;
  if (i>=usedchords || strcmp(s,used[i])!=0)
    strcpy(used[usedchords++],s);
}

void convert_chord(char *s, char *d)
{
  while (*s)
  {
    if (*s=='#')
      *d++='s';
    else if (*s==' ')
      d++;
    else if (*s=='\\' || *s=='/')
      *d++='-';
    else if (*s==':' || *s=='(' || *s==')')
      *d++='_';
    else
      *d++=*s;
    s++;
  };
  *d='\0';
}

void print_chords(void)
{
  int i,j;
  char s[256];

  qsort(used,usedchords,CHORDLEN,strcmp);

  i=0;
  while (i<usedchords)
  {
    fprintf(out,"<P><TABLE>\n<TR>");
    j=0;
    while (j<CHORDSPERLINE && i+j<usedchords)
    {
      fprintf(out,"<TD ALIGN=CENTER>%s",used[i+j]);
      j++;
    };
    fprintf(out,"</TR>\n<TR>");
    j=0;
    while (j<CHORDSPERLINE && i+j<usedchords)
    {
      convert_chord(used[i+j],s);
      if (find_chord(used[i+j])<0)
        fprintf(out,"<TD><IMG SRC=\"%s%c%s.gif\">",chorddir,DIRSEP,s);
      else
        fprintf(out,"<TD><IMG SRC=\"%s.chords%d%c%s.gif\">",filename,song,DIRSEP,s);
      j++;
    };
    fprintf(out,"</TR>\n</TABLE>\n");
    i+=CHORDSPERLINE;
  }
}

void make_chord(char *s)
{
  char n[256],cmd[256],*c,*e;

  s+=7;
  while (*s==' ')
    s++;
  c=s;
  while (*c!=' ' && *c!=':' && *c!='\0')
    c++;
  if (*c!='\0')
    *c++='\0';
  strcpy(defined[defchords++],s);
  convert_chord(s,n);
  if ((e=strstr(c,"base-fret")))
    memset(e,' ',9);
  if ((e=strstr(c,"frets")))
    memset(e,' ',5);
  if ((e=strchr(c,'}')))
    *e='\0';
  sprintf(cmd,"chord2pbm %s | ppmtogif -transparent rgb:ff/ff/ff > %s.chords%d%c%s.gif",c,filename,song,DIRSEP,n);
  fprintf(stderr,"executing %s\n",cmd);
  system(cmd);
}

int find_chord(char *s)
{
  int i;

  i=0;
  while (i<defchords && strcmp(s,defined[i])!=0)
    i++;

  return (i<defchords) ? i : -1;
}
